#（1）导入外部库和Fashion-MNIST数据集
import tensorflow as tf
import matplotlib.pyplot as plt
from sklearn import metrics
fashion_mnist = tf.keras.datasets.fashion_mnist
(X_train, y_train), (X_test, y_test) = fashion_mnist.load_data()

#（2）查看训练集和测试集的形状
print('The shape of train data=',X_train.shape)
print('The shape of y_train:',y_train.shape)
print('The shape of test data=',X_test.shape)
print('The shape of y_test:',y_test.shape)

#（3）建立映射表
class_names = ['T-shirt/top', 'Trouser', 'Pullover', 'Dress', 'Coat', \
               'Sandal', 'Shirt', 'Sneaker', 'Bag', 'Ankle boot']

#（4）显示训练集的前20个图像和标签
plt.figure(figsize=(10,9))
num=20
for i in range(0, num):
    plt.subplot(4,5,i+1)
    plt.imshow(X_train[i], cmap='gray')
    plt.xticks([])
    plt.yticks([])
    plt.title("True="+str(class_names[y_train[i]]))
plt.show()

#（5）利用reshape函数转换数字图像
X_train_reshape = X_train.reshape(X_train.shape[0],28,28,1)
X_test_reshape = X_test.reshape(X_test.shape[0],28,28,1)

#（6）查看经过reshape之后训练集和测试集的形状
print('The shape of train reshape data=',X_train_reshape.shape)
print('The shape of y_train:',y_train.shape)
print('The shape of test reshape data=',X_test_reshape.shape)
print('The shape of y_train:',y_test.shape)

#（7）归一化数字图像
X_train_norm, X_test_norm = X_train_reshape / 255.0, X_test_reshape / 255.0

#（8）构建Sequential模型
model = tf.keras.models.Sequential()

#（9）构造卷积神经网络
#构建卷积层
model.add(tf.keras.layers.Conv2D(32, (3, 3), activation='relu', input_shape=(28, 28, 1)))
#构建池化层
model.add(tf.keras.layers.MaxPooling2D((2, 2)))
#构建Dropout层
model.add(tf.keras.layers.Dropout(rate=0.2))

#（10）构建全连接层
model.add(tf.keras.layers.Flatten())
model.add(tf.keras.layers.Dense(50, activation='relu'))
model.add(tf.keras.layers.Dense((10),activation='softmax'))

#（11）打印模型的概况
print(model.summary())

#（12）模型编译
model.compile(optimizer='adam',
              loss='sparse_categorical_crossentropy',
              metrics=['accuracy'])

#（13）模型训练 
history = model.fit(X_train_norm, y_train, validation_split=0.2, epochs=10, verbose=1)

#（14）模型评估
model.evaluate(X_test_norm, y_test, verbose=1)

#（15）模型预测
prediction=model.predict_classes(X_test_norm)

#（16）显示测试集的前20个图像的预测类别和真实类别
plt.figure(figsize=(18,9))
num=20
for i in range(0, num):
    plt.subplot(4,5,i+1)
    plt.imshow(X_test[i], cmap='gray')
    plt.xticks([])
    plt.yticks([])
    plt.title("Predict="+str(class_names[prediction[i]]) +";True=" + str(class_names[y_test[i]]))
plt.show()

#（17）计算混淆矩阵
print('Confusion Matrix：')
print(metrics.confusion_matrix(y_test, prediction))
