#（1）导入外部库和CIFAR10数据集
import tensorflow as tf
import matplotlib.pyplot as plt
from sklearn import metrics
cifar10 = tf.keras.datasets.cifar10
(X_train, y_train), (X_test, y_test) = cifar10.load_data()

#（2）查看训练集和测试集的形状
print('The shape of train data=',X_train.shape)
print('The shape of y_train:',y_train.shape)
print('The shape of test data=',X_test.shape)
print('The shape of y_test:',y_test.shape)

#（3）建立映射表
class_names = ['airplane', 'automobile', 'bird', 'cat', 'deer',
               'dog', 'frog', 'horse', 'ship', 'truck']

#（4）显示训练集的前20个图像和标签
plt.figure(figsize=(10,9))
num=20
for i in range(0, num):
    plt.subplot(4,5,i+1)
    plt.imshow(X_train[i])
    plt.xticks([])
    plt.yticks([])
    plt.title("True="+str(class_names[y_train[i][0]]))
plt.show()

#（5）归一化数字图像
X_train_norm, X_test_norm = X_train / 255.0, X_test / 255.0

#（6）构建Sequential模型
model = tf.keras.models.Sequential()

#（7）构造卷积神经网络
#构建卷积层
model.add(tf.keras.layers.Conv2D(32, (3, 3), activation='relu', input_shape=(32, 32, 3)))
#构建池化层
model.add(tf.keras.layers.MaxPooling2D((2, 2)))
#构建卷积层
model.add(tf.keras.layers.Conv2D(64, (3, 3), activation='relu',))
#构建池化层
model.add(tf.keras.layers.MaxPooling2D((2, 2)))
#构建Dropout层
model.add(tf.keras.layers.Dropout(rate=0.2))

#（8）构建全连接层
model.add(tf.keras.layers.Flatten())
model.add(tf.keras.layers.Dense(80, activation='relu'))
model.add(tf.keras.layers.Dense((10),activation='softmax'))

#（9）打印模型的概况
print(model.summary())

#（10）模型编译
model.compile(optimizer='adam',
              loss='sparse_categorical_crossentropy',
              metrics=['accuracy'])

#（11）模型训练 
history = model.fit(X_train_norm, y_train, validation_split=0.1, epochs=20, verbose=1)

#（12）模型评估
model.evaluate(X_test_norm, y_test, verbose=1)

#（13）模型预测
prediction=model.predict_classes(X_test_norm)

#（14）显示测试集的前20个图像的预测类别和真实类别
plt.figure(figsize=(18,9))
num=20
for i in range(0, num):
    plt.subplot(4,5,i+1)
    plt.imshow(X_test[i], cmap='gray')
    plt.xticks([])
    plt.yticks([])
    plt.title("Predict="+str(class_names[prediction[i]]) +";True=" + str(class_names \
[y_test[i][0]))
plt.show()

#（15）计算混淆矩阵
print('Confusion Matrix：')
print(metrics.confusion_matrix(y_test, prediction))
